#ifndef AMDIS_STLVECTOR_H
#define AMDIS_STLVECTOR_H

#include <vector>
#include "CreatorInterface.h"
#include "MemoryManager.h"
#include "DOFMatrix.h"

namespace AMDiS {

  class StlVectorCreator : public CreatorInterface< ::std::vector<double> > {
  public:
    MEMORY_MANAGED(StlVectorCreator);

    StlVectorCreator(int size)
      : size_(size)
    {};

    virtual ~StlVectorCreator() {};

    ::std::vector<double> *create() {
      ::std::vector<double> *vec = new ::std::vector<double>(size_, 0.0);

      return vec;
    };

    void free(::std::vector<double> *vec) { 
      delete vec;
    };    

  private:
    int size_;
  };

  double norm(::std::vector<double> *vec);

  inline int size(::std::vector<double> *vec)
  {
    return vec->size();
  }

  void setValue(::std::vector<double>& x, double value);
   
  const ::std::vector<double>& operator*=(::std::vector<double>& x, double scal);

  const ::std::vector<double>& operator+=(::std::vector<double>& x1,
					  const ::std::vector<double>& x2);

  double operator*(::std::vector<double>& x1, ::std::vector<double>& x2);

  void axpy(double alpha, const ::std::vector<double>& x, ::std::vector<double>& y);

  void xpay(double alpha, const ::std::vector<double>& x, ::std::vector<double>& y);

  int logToPhysIndex(::std::vector< ::std::vector<MatEntry> > *m, int a, int b);

  void print(::std::vector<double> *vec);
}

#endif
